/* edit-acrostic-answer.c
 *
 * Copyright 2024 Tanmay Patil <tanmaynpatil105@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "crosswords-config.h"
#include <glib/gi18n-lib.h>
#include "edit-acrostic-answer.h"

enum
{
  ANSWER_SELECTION_TOGGLED,
  N_SIGNALS
};

static guint obj_signals[N_SIGNALS] = { 0 };


struct _EditAcrosticAnswer
{
  GtkWidget parent_instance;

  GtkWidget *first_letter;
  GtkWidget *answer;
  GtkWidget *check_button;
};

G_DEFINE_TYPE (EditAcrosticAnswer, edit_acrostic_answer, GTK_TYPE_WIDGET);

static void edit_acrostic_answer_init       (EditAcrosticAnswer      *self);
static void edit_acrostic_answer_class_init (EditAcrosticAnswerClass *klass);
static void edit_acrostic_answer_dispose    (GObject                 *object);
static void selection_toggled_cb            (EditAcrosticAnswer      *self,
                                             GtkCheckButton          *button);

static void
edit_acrostic_answer_init (EditAcrosticAnswer *self)
{
  gtk_widget_init_template (GTK_WIDGET (self));
}

static void
edit_acrostic_answer_class_init (EditAcrosticAnswerClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

  object_class->dispose = edit_acrostic_answer_dispose;

  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Crosswords/edit-acrostic-answer.ui");

  gtk_widget_class_bind_template_child (widget_class, EditAcrosticAnswer, first_letter);
  gtk_widget_class_bind_template_child (widget_class, EditAcrosticAnswer, answer);
  gtk_widget_class_bind_template_child (widget_class, EditAcrosticAnswer, check_button);

  gtk_widget_class_bind_template_callback (widget_class, selection_toggled_cb);

  obj_signals [ANSWER_SELECTION_TOGGLED] =
    g_signal_new ("answer-selection-toggled",
                  EDIT_TYPE_ACROSTIC_ANSWER,
                  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                  0,
                  NULL, NULL,
                  NULL,
                  G_TYPE_NONE, 1,
                  G_TYPE_BOOLEAN);


  gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BOX_LAYOUT);
}

static void
edit_acrostic_answer_dispose (GObject *object)
{
  GtkWidget *child;

  while ((child = gtk_widget_get_first_child (GTK_WIDGET (object))))
    gtk_widget_unparent (child);

  G_OBJECT_CLASS (edit_acrostic_answer_parent_class)->dispose (object);
}

static void
selection_toggled_cb (EditAcrosticAnswer *self,
                      GtkCheckButton     *button)
{
  gboolean active;

  active = gtk_check_button_get_active (button);

  g_signal_emit (self,
                 obj_signals[ANSWER_SELECTION_TOGGLED],
                 0, active);
}

GtkWidget *
edit_acrostic_answer_new (const char *str)
{
  EditAcrosticAnswer *answer;

  answer = g_object_new (EDIT_TYPE_ACROSTIC_ANSWER, NULL);

  if (str != NULL)
    edit_acrostic_answer_set_str (answer, str);

  return GTK_WIDGET (answer);
}

void
edit_acrostic_answer_set_str (EditAcrosticAnswer *self,
                              const char         *str)
{
  gchar first_letter[2] = { 0 };

  first_letter[0] = str[0];

  gtk_label_set_text (GTK_LABEL (self->first_letter), first_letter);
  gtk_label_set_text (GTK_LABEL (self->answer), str + 1);
}

void
edit_acrostic_answer_set_selected (EditAcrosticAnswer *self,
                                   gboolean            selected)
{
  gtk_check_button_set_active (GTK_CHECK_BUTTON (self->check_button),
                               selected);
}
