/* puzzle-render.h - Render puzzles using Cairo
 *
 * Copyright 2025 Toluwaleke Ogundipe <anonymoux47@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <cairo.h>
#include <libipuz/libipuz.h>

#include "grid-layout.h"


typedef enum {
  RENDER_COLOR_SCHEME_LIGHT,
  RENDER_COLOR_SCHEME_DARK,
} RenderColorScheme;


typedef struct
{
  double x, y;
  double width, height;
} RenderViewport;


void render_layout             (cairo_t              *cr,
                                const RenderViewport *viewport,
                                GridLayout           *layout,
                                RenderColorScheme     color_scheme);
void render_layout_overlays    (cairo_t              *cr,
                                const RenderViewport *viewport,
                                GridLayout           *layout,
                                RenderColorScheme     color_scheme);
void render_puzzle             (cairo_t              *cr,
                                const RenderViewport *viewport,
                                IpuzPuzzle           *puzzle,
                                RenderColorScheme     color_scheme);
void render_puzzle_with_config (cairo_t              *cr,
                                const RenderViewport *viewport,
                                IpuzPuzzle           *puzzle,
                                RenderColorScheme     color_scheme,
                                LayoutConfig          config);
